@@ -2,7 +2,7 @@ package Catalyst::DispatchType::Regex;
 
 use strict;
 use base qw/Catalyst::DispatchType::Path/;
-use Text::ASCIITable;
+use Text::SimpleTable;
 
 =head1 NAME
 
@@ -16,28 +16,27 @@ See L<Catalyst>.
 
 =head1 METHODS
 
-=over 4
+=head2 $self->list($c)
 
-=item $self->list($c)
+Output a table of all regex actions, and their private equivalent.
 
 =cut
 
 sub list {
     my ( $self, $c ) = @_;
-    my $re = Text::ASCIITable->new;
-    $re->setCols( 'Regex', 'Private' );
-    $re->setColWidth( 'Regex',   36, 1 );
-    $re->setColWidth( 'Private', 37, 1 );
+    my $re = Text::SimpleTable->new( [ 35, 'Regex' ], [ 36, 'Private' ] );
     for my $regex ( @{ $self->{compiled} } ) {
-        my $compiled = $regex->{re};
-        my $action   = $regex->{action};
-        $re->addRow( $compiled, "/$action" );
+        my $action = $regex->{action};
+        $re->row( $regex->{path}, "/$action" );
     }
     $c->log->debug( "Loaded Regex actions:\n" . $re->draw )
-      if ( @{ $re->{tbl_rows} } );
+      if ( @{ $self->{compiled} } );
 }
 
-=item $self->match( $c, $path )
+=head2 $self->match( $c, $path )
+
+Check path against compiled regexes, and set action to any matching
+action. Returns 1 on success and 0 on failure.
 
 =cut
 
@@ -50,6 +49,7 @@ sub match {
 
     foreach my $compiled ( @{ $self->{compiled} || [] } ) {
         if ( my @snippets = ( $path =~ $compiled->{re} ) ) {
+            next unless $compiled->{action}->match($c);
             $c->req->action( $compiled->{path} );
             $c->req->match($path);
             $c->req->snippets( \@snippets );
@@ -62,31 +62,47 @@ sub match {
     return 0;
 }
 
-=item $self->register( $c, $action )
+=head2 $self->register( $c, $action )
+
+Registers one or more regex actions for an action object.\
+Also registers them as literal paths.
+
+Returns 1 on if any regexps were registered.
 
 =cut
 
 sub register {
     my ( $self, $c, $action ) = @_;
-    my $attrs = $action->attributes;
-    my @register = map { @{ $_ || [] } } @{$attrs}{ 'Regex', 'Regexp' };
+    my $attrs    = $action->attributes;
+    my @register = @{ $attrs->{'Regex'} || [] };
+
     foreach my $r (@register) {
-        unless ($r =~ /^\^/) {     # Relative regex
-            $r = '^'.$action->namespace.'/'.$r;
-        }
-        $self->{paths}{$r} = $action;    # Register path for superclass
-        push(
-            @{ $self->{compiled} },      # and compiled regex for us
-            {
-                re     => qr#$r#,
-                action => $action,
-                path   => $r,
-            }
-        );
+        $self->register_path( $c, $r, $action );
+        $self->register_regex( $c, $r, $action );
     }
+
+    return 1 if @register;
+    return 0;
 }
 
-=back
+=head2 $self->register_regex($c, $re, $action)
+
+Register an individual regex on the action. Usually called from the 
+register action.
+
+=cut
+
+sub register_regex {
+    my ( $self, $c, $re, $action ) = @_;
+    push(
+        @{ $self->{compiled} },    # and compiled regex for us
+        {
+            re     => qr#$re#,
+            action => $action,
+            path   => $re,
+        }
+    );
+}
 
 =head1 AUTHOR
 
