@@ -30,13 +30,8 @@ See L<Catalyst>.
 
 =head1 METHODS
 
-=over 4
 
-=item $self->finalize_output
-
-<obsolete>, see finalize_body
-
-=item $self->finalize_body($c)
+=head2 $self->finalize_body($c)
 
 Finalize body.  Prints the response output.
 
@@ -44,19 +39,22 @@ Finalize body.  Prints the response output.
 
 sub finalize_body {
     my ( $self, $c ) = @_;
-    if ( ref $c->response->body && $c->response->body->can('read') ) {
-        while ( !$c->response->body->eof() ) {
-            $c->response->body->read( my $buffer, $CHUNKSIZE );
-            $self->write( $c, $buffer );
+    my $body = $c->response->body;
+    if ( ref $body && ( $body->can('read') || ref($body) eq 'GLOB' ) ) {
+        while ( !eof $body ) {
+            read $body, my ($buffer), $CHUNKSIZE;
+            last unless $self->write( $c, $buffer );
         }
-        $c->response->body->close();
+        close $body;
     }
     else {
-        $self->write( $c, $c->response->body );
+        $self->write( $c, $body );
     }
 }
 
-=item $self->finalize_cookies($c)
+=head2 $self->finalize_cookies($c)
+
+Create CGI::Cookies from $c->res->cookies, and set them as response headers.
 
 =cut
 
@@ -78,12 +76,16 @@ sub finalize_cookies {
         push @cookies, $cookie->as_string;
     }
 
-    if (@cookies) {
-        $c->res->headers->push_header( 'Set-Cookie' => join ',', @cookies );
+    for my $cookie (@cookies) {
+        $c->res->headers->push_header( 'Set-Cookie' => $cookie );
     }
 }
 
-=item $self->finalize_error($c)
+=head2 $self->finalize_error($c)
+
+Output an apropriate error message, called if there's an error in $c
+after the dispatch has finished. Will output debug messages if Catalyst
+is in debug mode, or a `please come back later` message otherwise.
 
 =cut
 
@@ -98,12 +100,15 @@ sub finalize_error {
 
         # For pretty dumps
         local $Data::Dumper::Terse = 1;
-        $error = join '',
-          map { '<p><code class="error">' . encode_entities($_) . '</code></p>' }
-          @{ $c->error };
+        $error = join '', map {
+                '<p><code class="error">'
+              . encode_entities($_)
+              . '</code></p>'
+        } @{ $c->error };
         $error ||= 'No output';
+        $error = qq{<pre wrap="">$error</pre>};
         $title = $name = "$name on Catalyst $Catalyst::VERSION";
-        $name = "<h1>$name</h1>";
+        $name  = "<h1>$name</h1>";
 
         # Don't show context in the dump
         delete $c->req->{_context};
@@ -127,7 +132,7 @@ sub finalize_error {
             push @infos, sprintf <<"EOF", $name, $value;
 <h2><a href="#" onclick="toggleDump('dump_$i'); return false">%s</a></h2>
 <div id="dump_$i">
-    <pre>%s</pre>
+    <pre wrap="">%s</pre>
 </div>
 EOF
             $i++;
@@ -141,12 +146,10 @@ EOF
 <pre>
 (en) Please come back later
 (de) Bitte versuchen sie es spaeter nocheinmal
-(nl) Gelieve te komen later terug
+(at) Konnten's bitt'schoen spaeter nochmal reinschauen
 (no) Vennligst prov igjen senere
-(fr) Veuillez revenir plus tard
-(es) Vuelto por favor mas adelante
-(pt) Voltado por favor mais tarde
-(it) Ritornato prego più successivamente
+(dk) Venligst prov igen senere
+(pl) Prosze sprobowac pozniej
 </pre>
 
         $name = '';
@@ -190,7 +193,6 @@ EOF
             border: 1px solid #aaa;
             padding: 4px;
             margin: 10px;
-            -moz-border-radius: 10px;
         }
         div.error {
             background-color: #977;
@@ -198,7 +200,6 @@ EOF
             padding: 8px;
             margin: 4px;
             margin-bottom: 10px;
-            -moz-border-radius: 10px;
         }
         div.infos {
             background-color: #797;
@@ -206,20 +207,17 @@ EOF
             padding: 8px;
             margin: 4px;
             margin-bottom: 10px;
-            -moz-border-radius: 10px;
         }
         div.name {
             background-color: #779;
             border: 1px solid #557;
             padding: 8px;
             margin: 4px;
-            -moz-border-radius: 10px;
         }
         code.error {
             display: block;
             margin: 1em 0;
             overflow: auto;
-            white-space: pre;
         }
         div.name h1, div.error p {
             margin: 0;
@@ -235,6 +233,15 @@ EOF
             font-size: medium;
             font-weight: normal;
         }
+        /* from http://users.tkk.fi/~tkarvine/linux/doc/pre-wrap/pre-wrap-css3-mozilla-opera-ie.html */
+        /* Browser specific (not valid) styles to make preformatted text wrap */
+        pre { 
+            white-space: pre-wrap;       /* css-3 */
+            white-space: -moz-pre-wrap;  /* Mozilla, since 1999 */
+            white-space: -pre-wrap;      /* Opera 4-6 */
+            white-space: -o-pre-wrap;    /* Opera 7 */
+            word-wrap: break-word;       /* Internet Explorer 5.5+ */
+        }
     </style>
 </head>
 <body>
@@ -246,15 +253,23 @@ EOF
 </body>
 </html>
 
+
+    # Trick IE
+    $c->res->{body} .= ( ' ' x 512 );
+
+    # Return 500
+    $c->res->status(500);
 }
 
-=item $self->finalize_headers($c)
+=head2 $self->finalize_headers($c)
+
+Abstract method, allows engines to write headers to response
 
 =cut
 
 sub finalize_headers { }
 
-=item $self->finalize_read($c)
+=head2 $self->finalize_read($c)
 
 =cut
 
@@ -264,7 +279,9 @@ sub finalize_read {
     undef $self->{_prepared_read};
 }
 
-=item $self->finalize_uploads($c)
+=head2 $self->finalize_uploads($c)
+
+Clean up after uploads, deleting temp files.
 
 =cut
 
@@ -281,7 +298,9 @@ sub finalize_uploads {
     }
 }
 
-=item $self->prepare_body($c)
+=head2 $self->prepare_body($c)
+
+sets up the L<Catalyst::Request> object body using L<HTTP::Body>
 
 =cut
 
@@ -293,16 +312,28 @@ sub prepare_body {
 
     unless ( $c->request->{_body} ) {
         $c->request->{_body} = HTTP::Body->new( $type, $self->read_length );
+        $c->request->{_body}->{tmpdir} = $c->config->{uploadtmp}
+          if exists $c->config->{uploadtmp};
     }
 
     if ( $self->read_length > 0 ) {
         while ( my $buffer = $self->read($c) ) {
             $c->prepare_body_chunk($buffer);
         }
+
+        # paranoia against wrong Content-Length header
+        my $remaining = $self->read_length - $self->read_position;
+        if ( $remaining > 0 ) {
+            $self->finalize_read($c);
+            Catalyst::Exception->throw(
+                "Wrong Content-Length value: " . $self->read_length );
+        }
     }
 }
 
-=item $self->prepare_body_chunk($c)
+=head2 $self->prepare_body_chunk($c)
+
+Add a chunk to the request body.
 
 =cut
 
@@ -312,7 +343,9 @@ sub prepare_body_chunk {
     $c->request->{_body}->add($chunk);
 }
 
-=item $self->prepare_body_parameters($c)
+=head2 $self->prepare_body_parameters($c)
+
+Sets up parameters from body. 
 
 =cut
 
@@ -321,13 +354,17 @@ sub prepare_body_parameters {
     $c->request->body_parameters( $c->request->{_body}->param );
 }
 
-=item $self->prepare_connection($c)
+=head2 $self->prepare_connection($c)
+
+Abstract method implemented in engines.
 
 =cut
 
 sub prepare_connection { }
 
-=item $self->prepare_cookies($c)
+=head2 $self->prepare_cookies($c)
+
+Parse cookies from header. Sets a L<CGI::Cookie> object.
 
 =cut
 
@@ -339,13 +376,15 @@ sub prepare_cookies {
     }
 }
 
-=item $self->prepare_headers($c)
+=head2 $self->prepare_headers($c)
 
 =cut
 
 sub prepare_headers { }
 
-=item $self->prepare_parameters($c)
+=head2 $self->prepare_parameters($c)
+
+sets up parameters from query and post parameters.
 
 =cut
 
@@ -372,15 +411,19 @@ sub prepare_parameters {
     }
 }
 
-=item $self->prepare_path($c)
+=head2 $self->prepare_path($c)
+
+abstract method, implemented by engines.
 
 =cut
 
 sub prepare_path { }
 
-=item $self->prepare_request($c)
+=head2 $self->prepare_request($c)
 
-=item $self->prepare_query_parameters($c)
+=head2 $self->prepare_query_parameters($c)
+
+process the query string and extract query parameters.
 
 =cut
 
@@ -398,7 +441,9 @@ sub prepare_query_parameters {
     }
 }
 
-=item $self->prepare_read($c)
+=head2 $self->prepare_read($c)
+
+prepare to read from the engine.
 
 =cut
 
@@ -409,13 +454,15 @@ sub prepare_read {
     $self->read_position(0);
 }
 
-=item $self->prepare_request(@arguments)
+=head2 $self->prepare_request(@arguments)
+
+Populate the context object from the request object.
 
 =cut
 
 sub prepare_request { }
 
-=item $self->prepare_uploads($c)
+=head2 $self->prepare_uploads($c)
 
 =cut
 
@@ -444,13 +491,15 @@ sub prepare_uploads {
     }
 }
 
-=item $self->prepare_write($c)
+=head2 $self->prepare_write($c)
+
+Abstract method. Implemented by the engines.
 
 =cut
 
 sub prepare_write { }
 
-=item $self->read($c, [$maxlength])
+=head2 $self->read($c, [$maxlength])
 
 =cut
 
@@ -483,7 +532,7 @@ sub read {
     }
 }
 
-=item $self->read_chunk($c, $buffer, $length)
+=head2 $self->read_chunk($c, $buffer, $length)
 
 Each engine inplements read_chunk as its preferred way of reading a chunk
 of data.
@@ -492,22 +541,26 @@ of data.
 
 sub read_chunk { }
 
-=item $self->read_length
+=head2 $self->read_length
 
 The length of input data to be read.  This is obtained from the Content-Length
 header.
 
-=item $self->read_position
+=head2 $self->read_position
 
 The amount of input data that has already been read.
 
-=item $self->run($c)
+=head2 $self->run($c)
+
+Start the engine. Implemented by the various engine classes.
 
 =cut
 
 sub run { }
 
-=item $self->write($c, $buffer)
+=head2 $self->write($c, $buffer)
+
+Writes the buffer to the client. Can only be called once for a request.
 
 =cut
 
@@ -522,7 +575,9 @@ sub write {
     print STDOUT $buffer;
 }
 
-=back
+=head2 $self->finalize_output
+
+<obsolete>, see finalize_body
 
 =head1 AUTHORS
 
